@@ -3,7 +3,7 @@ from __future__ import unicode_literals, print_function
 
 import itertools as it, operator as op, functools as ft
 from collections import defaultdict, OrderedDict, namedtuple
-import os, sys, io
+import os, sys, io, re
 
 import yaml
 
@@ -15,6 +15,7 @@ class PrettyYAMLDumper(yaml.dumper.SafeDumper):
 	def __init__(self, *args, **kws):
 		self.pyaml_force_embed = kws.pop('force_embed', False)
 		self.pyaml_string_val_style = kws.pop('string_val_style', None)
+		self.pyaml_sort_dicts = kws.pop('sort_dicts', True)
 		return super(PrettyYAMLDumper, self).__init__(*args, **kws)
 
 	def represent_odict(dumper, data):
@@ -38,6 +39,10 @@ class PrettyYAMLDumper(yaml.dumper.SafeDumper):
 		elif callable(getattr(data, 'tolist', None)): return dumper.represent_data(data.tolist())
 		return super(PrettyYAMLDumper, dumper).represent_undefined(data)
 
+	def represent_dict(dumper, data):
+		if not dumper.pyaml_sort_dicts: return dumper.represent_odict(data)
+		return super(PrettyYAMLDumper, dumper).represent_dict(data)
+
 	def serialize_node(self, node, parent, index):
 		if self.pyaml_force_embed: self.serialized_nodes.clear()
 		return super(PrettyYAMLDumper, self).serialize_node(node, parent, index)
@@ -70,6 +75,7 @@ class PrettyYAMLDumper(yaml.dumper.SafeDumper):
 					self.anchor_node(key)
 					self.anchor_node(value, hint=hint+[key])
 
+PrettyYAMLDumper.add_representer(dict, PrettyYAMLDumper.represent_dict)
 PrettyYAMLDumper.add_representer(defaultdict, PrettyYAMLDumper.represent_dict)
 PrettyYAMLDumper.add_representer(OrderedDict, PrettyYAMLDumper.represent_odict)
 PrettyYAMLDumper.add_representer(set, PrettyYAMLDumper.represent_list)
@@ -99,7 +105,12 @@ class UnsafePrettyYAMLDumper(PrettyYAMLDumper):
 			self.states.append(self.expect_block_sequence_item)
 			self.expect_node(sequence=True)
 
-	def choose_scalar_style(self):
+	def check_simple_key(self):
+		res = super(UnsafePrettyYAMLDumper, self).check_simple_key()
+		self.analysis.allow_flow_plain = False # not always-set for keys in newer pyyaml
+		return res
+
+	def choose_scalar_style(self, _re1=re.compile(':(\s|$)')):
 		is_dict_key = self.states[-1] == self.expect_block_mapping_simple_value
 		if is_dict_key:
 			# Don't mess-up (replace) styles for dict keys, if possible
@@ -107,8 +118,13 @@ class UnsafePrettyYAMLDumper(PrettyYAMLDumper):
 		else:
 			# Make sure we don't create "key: null" mapping accidentally
 			if self.event.value.endswith(':'): self.event.style = "'"
-		return super(UnsafePrettyYAMLDumper, self).choose_scalar_style()\
-			if self.event.style != 'plain' else ("'" if ' ' in self.event.value else None)
+		if self.event.style != 'plain':
+			return super(UnsafePrettyYAMLDumper, self).choose_scalar_style()
+		s = self.event.value
+		if s.startswith('- ') or _re1.search(s): return "'"
+		if self.analysis and not self.analysis.allow_flow_plain:
+			# Can be a mapping key - disallow spaces in those
+			if ' ' in s: return "'"
 
 	def represent_stringish(dumper, data):
 		# Will crash on bytestrings with weird chars in them,
@@ -126,7 +142,7 @@ class UnsafePrettyYAMLDumper(PrettyYAMLDumper):
 		style = dumper.pyaml_string_val_style
 		if not style:
 			style = 'plain'
-			if '\n' in data or not data or data == '-' or data[0] in '!&*[':
+			if '\n' in data or not data or data == '-' or data[0] in '!&*[' or '#' in data:
 				style = 'literal'
 				if '\n' in data[:-1]:
 					for line in data.splitlines():
@@ -157,7 +173,7 @@ def dump_add_vspacing(buff, vspacing):
 		level = 0
 		line = line.decode('utf-8')
 		result.append(line)
-		if ':' in line:
+		if ':' in line or re.search(r'---(\s*$|\s)', line):
 			while line.startswith('  '):
 				level, line = level + 1, line[2:]
 			if len(vspacing) > level and len(result) != 1:
@@ -168,12 +184,18 @@ def dump_add_vspacing(buff, vspacing):
 	buff.write(''.join(result).encode('utf-8'))
 
 
-def dump( data, dst=unicode, safe=False,
-		force_embed=False, vspacing=None, string_val_style=None, **pyyaml_kws ):
+def dump_all(data, *dump_args, **dump_kws):
+	return dump(data, *dump_args, multiple_docs=True, **dump_kws)
+
+def dump( data, dst=unicode, safe=False, force_embed=False, vspacing=None,
+		string_val_style=None, sort_dicts=True, multiple_docs=False, **pyyaml_kws ):
 	buff = io.BytesIO()
 	Dumper = PrettyYAMLDumper if safe else UnsafePrettyYAMLDumper
-	Dumper = ft.partial(Dumper, force_embed=force_embed, string_val_style=string_val_style)
-	yaml.dump_all( [data], buff, Dumper=Dumper,
+	Dumper = ft.partial( Dumper,
+		force_embed=force_embed, string_val_style=string_val_style, sort_dicts=sort_dicts )
+	if not multiple_docs: data = [data]
+	else: pyyaml_kws.setdefault('explicit_start', True)
+	yaml.dump_all( data, buff, Dumper=Dumper,
 		default_flow_style=False, allow_unicode=True, encoding='utf-8', **pyyaml_kws )
 
 	if vspacing is not None:
